


#include <p24F16KA101.h> 
#include "common.h" 


// ==============================================================================================
//   CONFIGS - FUNZIONANTI 
//   copiati da Configuration-bits dell' MPLAB-IDE  ( meglio imparare a impostarli nel codice )
// ==============================================================================================

// FNOSC    = FRCDIV + PLL
// IESO     = DISABLED
// POSCMOD  = HS
// OSCIOFNC = CLKO output disabled ( in ogni caso sarebbe disabilitato dato che si usa un oscillatore esterno )
// POSCFREQ = primary osc external clock frequency between 100 KHz and 8 MHz ( sembra non importante )
// FCKSM    = both Clock-switching and Fail-safe-clock-monitor DISABLED
// PWRTEN	= PowerUp timer ENABLED
// MCLRE	= MCLR pin disabled ( se enabled non usare low-voltage altrimenti si verifica un errore durante la programmazione )



// ==============================================================================================
//   CONFIGS
// ==============================================================================================




// --------------------------------------------------------------------------- FGS ( CODE PROTECT / WRITE PROTECT )
//
//   General Segment Code Protect:
//     GCP_ON               Enabled
//     GCP_OFF              Disabled
//
//   General Segment Write Protect Enable:
//     GWRP_ON              Enabled
//     GWRP_OFF             Disabled
//

// =============================================
//  Always CODE PROTECTED and WRITE PROTECTED 
// =============================================
_FGS( GCP_ON & GWRP_ON )
//_FGS( GCP_OFF & GWRP_OFF )




// --------------------------------------------------------------------------- FOSCSEL ( INITIAL OSCILLATOR / START UP )
//
//   Initial Oscillator Select:
//     FNOSC_FRC            Fast RC Oscillator (FRC)
//     FNOSC_FRCPLL         Fast RC Oscillator with PLL Module (FRCPLL)
//     FNOSC_PRI            Primary Oscillator (MS, HS, EC)
//     FNOSC_PRIPLL         Primary Oscillator with PLL Module (MSPLL, HSPLL, ECPLL)
//     FNOSC_SOSC           Secondary Oscillator (SOSC)
//     FNOSC_LPRC           Low Power RC Oscillator (LPRC)
//     FNOSC_LPFRC          Low Power Fast RC Oscillator With Postscaler - 500kHz (LPFRC)
//     FNOSC_FRCDIV         Fast RC Oscillator With Postscaler (FRCDIV)
//
//   Two Speed Start-up:
//     IESO_OFF             Disabled
//     IESO_ON              Enabled
//
_FOSCSEL(FNOSC_PRIPLL & IESO_OFF) 



// --------------------------------------------------------------------------- FOSC ( PRIMARY OSCILLATOR )
// 
//   Clock switching and Fail-Safe Clock monitor:
//     FCKSM_CSECME         Both Clock Switching and Monitor Enabled
//     FCKSM_CSECMD         Clock Switching Enabled, Clock Monitor Disabled
//     FCKSM_CSDCMD         Both Clock Switching and Monitor Disabled
//
//   Primary Oscillator Frequency Range:
//     POSCFREQ_LS          Low Speed (< 100kHz)
//     POSCFREQ_MS          Mid Speed (100kHz - 8MHz)
//     POSCFREQ_HS          High Speed (> 8MHz)
//
//   OSCO Pin Configuration:
//     OSCIOFNC_ON          OSCO Pin Has Digital I/O Function (RA3)
//     OSCIOFNC_OFF         OSCO Pin Has Clock Out Function (CLKO)
//
//   Oscillator Selection:
//     POSCMOD_EC           External clock
//     POSCMOD_XT           XT oscillator
//     POSCMOD_HS           HS oscillator
//     POSCMOD_NONE         Primary disabled     				 
//
_FOSC(FCKSM_CSDCMD & POSCFREQ_MS & OSCIOFNC_ON & POSCMOD_HS) 




// ---------------------------------------------------------------------------  FWDT ( WATCHDOG )
//
//   Watchdog Timer:
//     FWDTEN_OFF           Disabled
//     FWDTEN_ON            Enabled
//
//   Windowed WDT:
//     WINDIS_ON            Window Mode
//     WINDIS_OFF           Non-Window Mode
//
//   Watchdog prescaler:
//     FWPSA_PR32           1:32
//     FWPSA_PR128          1:128
//
//   Watchdog postscale:             with PR32    with PR128
//     WDTPS_PS1            1:1          1 ms        4 ms
//     WDTPS_PS2            1:2          2 ms        8 ms
//     WDTPS_PS4            1:4          4 ms       16 ms
//     WDTPS_PS8            1:8          8 ms       32 ms
//     WDTPS_PS16           1:16        16 ms       64 ms
//     WDTPS_PS32           1:32        32 ms      128 ms
//     WDTPS_PS64           1:64        64 ms      256 ms
//     WDTPS_PS128          1:128      128 ms      512 ms  
//     WDTPS_PS256          1:256      256 ms        1 sec
//     WDTPS_PS512          1:512      512 ms        2 sec
//     WDTPS_PS1024         1:1,024      1 sec       4 sec
//     WDTPS_PS2048         1:2,048      2 sec       8 sec
//     WDTPS_PS4096         1:4,096      4 sec      16 sec
//     WDTPS_PS8192         1:8,192      8 sec      32 sec
//     WDTPS_PS16384        1:16,384    16 sec      64 sec
//     WDTPS_PS32768        1:32,768    32 sec     128 sec
//
//_FWDT ( FWDTEN_ON & WINDIS_OFF & FWPSA_PR32 & WDTPS_PS128 )  // PR32 & PS128 = 128 msec
//_FWDT ( FWDTEN_ON & WINDIS_OFF & FWPSA_PR32 & WDTPS_PS512 )  // PR32 & PS128 = 512 msec
//_FWDT ( FWDTEN_ON & WINDIS_OFF & FWPSA_PR32 & WDTPS_PS1024 )  // PR32 & PS1024 = 1 sec
_FWDT ( FWDTEN_OFF & WINDIS_OFF & FWPSA_PR32 & WDTPS_PS1024 )  // OFF




// ---------------------------------------------------------------------------  FPOR ( MCLR / Power-Up / BOR )
//
//   Master Clear Enable:
//     MCLRE_OFF            MCLR Disaled, RA5 Enabled
//     MCLRE_ON             MCLR Enabled, RA5 Disabled
//
//   Brown Out Voltage:
//     BORV_LPBOR           Low Power BOR
//     BORV_27V             2.7V
//     BORV_20V             2.0V
//     BORV_18V             1.8V
//
//   Power Up Timer:
//     PWRTEN_OFF           Disabled
//     PWRTEN_ON            Enabled
//
//   Brown Out Reset:
//     BOREN_BOR0           Disabled in hardware, SBOREN bit disabled
//     BOREN_BOR1           Software controlled by SBOREN bit
//     BOREN_BOR2           Disabled only when in SLEEP, SBOREN bit disabled
//     BOREN_BOR3           Enabled in hardware, SBOREN bit disabled
//

_FPOR(MCLRE_OFF & PWRTEN_ON)
//_FPOR(MCLRE_OFF & PWRTEN_ON & BORV_27V)




// ---------------------------------------------------------------------------  FICD  ( ICS )
//
//   ICD pins select:
//     ICS_PGx3             EMUC/EMUD share PGC3/PGD3
//     ICS_PGx2             EMUC/EMUD share PGC2/PGD2
//     ICS_PGx1             EMUC/EMUD share PGC1/PGD1
//
_FICD(ICS_PGx2 )


// ---------------------------------------------------------------------------  FDS ( DEEP SLEEP )
//
//   Deep Sleep Watchdog Timer:
//     DSWDTEN_OFF          Disabled
//     DSWDTEN_ON           Enabled
//
//   Deep Sleep BOR:
//     DSBOREN_OFF          Disabled
//     DSBOREN_ON           Enabled
//
//   Deep Sleep Watchdog Oscillator Clock Select:
//     DSWDTOSC_SOSC        Secondary Oscillator (SOSC)
//     DSWDTOSC_LPRC        Low Power RC Oscillator (LPRC)
//
//   Deep Sleep Watchdog Postscale Select Bits:
//     DSWDTPS_DSWDTPS0     1:2 (2.1 ms)
//     DSWDTPS_DSWDTPS1     1:8 (8.3 ms)
//     DSWDTPS_DSWDTPS2     1:32 (33 ms)
//     DSWDTPS_DSWDTPS3     1:128 (132 ms)
//     DSWDTPS_DSWDTPS4     1:512 (528 ms)
//     DSWDTPS_DSWDTPS5     1:2048 (2.1 Seconds)
//     DSWDTPS_DSWDTPS6     1:8192 (8.5 Seconds)
//     DSWDTPS_DSWDTPS7     1:32,768 (34 Seconds)
//     DSWDTPS_DSWDTPS8     1:131,072 (135 Seconds)
//     DSWDTPS_DSWDTPS9     1:524,288 (9 Minutes)
//     DSWDTPS_DSWDTPSA     1:2,097,152 (36 Minutes)
//     DSWDTPS_DSWDTPSB     1:8,388,608 (2.4 Hours)
//     DSWDTPS_DSWDTPSC     1:33,554,432 (9.6 Hours)
//     DSWDTPS_DSWDTPSD     1:134,217,728 (38.5 Hours)
//     DSWDTPS_DSWDTPSE     1:536,870,912 (6.4 Days)
//     DSWDTPS_DSWDTPSF     1:2,147,483,648 (25.7 Days)
//
//_FDS( DSWDTEN_OFF & DSBOREN_OFF & DSWDTOSC_LPRC & DSWDTPS_DSWDTPS4 )
//_FDS( DSWDTEN_OFF & DSBOREN_OFF )
_FDS( DSWDTEN_OFF & DSBOREN_OFF )	// PPPPP



// ------------------------------------------------------------------- Pin types
enum PIN_TYPE
{
	UNUSED			= 0,

	DIG_OUT			= 1,
	PWM_8			= 2,
	PWM_16			= 3,
	SERVO_8			= 4,
	SERVO_16		= 5,

	DIG_IN			= 129,
	DIG_IN_PU		= 130,

	ADC_8 			= 131,
	ADC_16 			= 132,
	CAP_8 			= 133,
	CAP_16 			= 134,
	RES_8 			= 135,
	RES_16 			= 136,

	COUNTER 		= 140,
	COUNTER_PU 		= 141,

	FAST_COUNTER 	= 142,
	FAST_COUNTER_PU = 143,
	
	PERIOD 			= 144,
	PERIOD_PU 		= 145,

	USOUND_SENSOR	= 150,
	
	CAP_SENSOR 		= 160,
	
	I2C_SDA 		= 170,
	I2C_SCL 		= 171,
	
	MCA_8_BYTES		= 180,
	MCA_16_BYTES	= 181,
	MCA_32_BYTES	= 182,

	SNIFFER			= 190,
	
//	VREF_POS 		= 180,
//	VREF_NEG 		= 181,
//
//	UART_TX 		= 182,
//	UART_RX 		= 183,
//	UART_CTS 		= 184,
//	UART_RTS 		= 185,
//	I2C 			= 186,
};



// ==============================================================================================
//   INIT
// ==============================================================================================

#define INVALID_SLAVE_ID 255
#define INVALID_SLAVE_TYPE 255

#define MAX_BYTES_FROM_MASTER 56
#define MAX_BYTES_TO_MASTER 56

#define SLAVE_ON _RB15

#define LED     _RB8
#define LED_ON  _RB8 = 1;
#define LED_OFF _RB8 = 0;

// ------------------------------------------------------------------- Commands
enum CMD
{
	// ------------------------------------------ commands that master expands for the slaves
	CMD_RECOG_START 				= 254,
	CMD_RECOG 						= 253,
	CMD_FAST_DATA_EXCHANGE 			= 251,

    // ------------------------------------------ commands that master repeats to slaves
	CMD_SETUP_SLAVE_PINS			= 249,
	CMD_SET_MASTER_NAME				= 248,
	CMD_GET_MASTER_NAME				= 247,
	CMD_SEND_VALUES 				= 246,
	CMD_GET_VALUES 					= 245,
	CMD_SEND_BYTES 					= 244,
	CMD_GET_BYTES 					= 243,

	// ------------------------------------------ commands to master only  ( < 200 )
	CMD_SPEED 						= 199,
	CMD_TEST_1 						= 99,
	CMD_NOCMD 						= 0
};


// ------------------------------------------------------------------- STATE MACHINES
typedef enum
{
	WAITING_START,
	RECOGNIZING,	
} main_status;
main_status Status = WAITING_START;

typedef enum
{
	INITIALIZING,
	RUNNING,
	WAITING_EXTERNAL_SYNC_FOR_START,	
} capsensor_status;
capsensor_status CapStatus = INITIALIZING;




// -------------------------------------- Input Bits 
// -------------------------------------- Output Bits   
uint32 T2_CapSensorTime; 

// -------------------------------------- Generic vars
uint8 gSlaveId;

uint8 BytesFromMaster[MAX_BYTES_FROM_MASTER];
uint8 BytesToMaster[MAX_BYTES_TO_MASTER];

uint8 gResync;

uint8 PinType;

// ------------------------------------------------------------ UART
#define UART1_RX_FIFO_SIZE 64

volatile uint8 au8_rxFifo1[UART1_RX_FIFO_SIZE];
volatile uint16 u16_rxFifo1Head = 0;
volatile uint16 u16_rxFifo1Tail = 0;

uint8 RX_Error = 0;

// ------------------------------------------------------------ PROTOTYPES
void __delay32(uint32 cycles);
void LowPriorityActivities();
	

// *******************************************
//   INTERRUPTS
// *******************************************

// The following macros can be used to declare interrupt     
// service routines (ISRs). For example, to declare an ISR
// for the timer1 interrupt:     
//
// void _ISR _T1Interrupt(void);
//
// To declare an ISR for the SPI1 interrupt with fast
// context save:
// 
// void _ISRFAST _SPI1Interrupt(void);
//
// Note: ISRs will be installed into the interrupt vector
// tables automatically if the reserved names listed in the
// MPLAB C30 Compiler User's Guide (DS51284) are used.
//
//#define _ISR __attribute__((interrupt))
//#define _ISRFAST __attribute__((interrupt, shadow))


//void __attribute__((__interrupt__, __shadow__)) _T1Interrupt(void)
//void __attribute__((interrupt)) _T1Interrupt(void)


//void _ISR _T1Interrupt(void)


// ==============================================================================================
//   INTERRUPT - On-Change int handler
// ==============================================================================================

//void __attribute__((__interrupt__, __no_auto_psv__, __shadow__)) _CNInterrupt(void)
//{
//
//}


// ==============================================================================================
//   TIMER1 INT HANDLER
// ==============================================================================================

void __attribute__((__interrupt__, __no_auto_psv__, __shadow__)) _T1Interrupt(void)
{
	if (CapStatus == WAITING_EXTERNAL_SYNC_FOR_START)
	{
		T2CONbits.TON = 1;
	}
	else
	{
		//T1CONbits.TON = 0;
		T2_CapSensorTime = TMR2;
		T2_CapSensorTime += 65536 * (uint32)TMR3;
		
		CapStatus = INITIALIZING;
	}	
	_T1IE = 0;		// Disable Timer1 interrupt
}


// ==============================================================================================
//   DELAYS
// ==============================================================================================
//
#define FCY 32000000 / 2				// instruction frequency (FCY = FOSC / 2)
#define CYCLES_PER_MS (FCY / 1000) 		// instructions per millisecond 
#define CYCLES_PER_US (FCY / 1000000) 	// instructions per microsecond 	
//
void Delay_ms(uint32 t)
{	
	while(t--) __delay32(CYCLES_PER_MS);
}
void Delay_us(uint32 t)
{	
	__delay32(CYCLES_PER_US * (t - 3) + 5);
}


// ==============================================================================================
//   UTILITY FUNCTIONS
// ==============================================================================================

uint8 gCrc;
void InitCrc()
{
	gCrc = 0;
}	
uint8 AppendArrayToCrc(uint8* bytes, uint16 nBytes)
{
	uint16 i;
    for (i = 0; i < nBytes; i++)
    {
    	gCrc = (gCrc ^ bytes[i]) + 1;
    }
   	return gCrc;
}
uint8 AppendByteToCrc(uint8 new_byte)
{
    gCrc = (gCrc ^ new_byte) + 1;
   	return gCrc;
}


void EmptyRxFifo()
{
	uint8 Dummy8;
	Dummy8 = U1RXREG;
	Dummy8 = U1RXREG;
	Dummy8 = U1RXREG;
	Dummy8 = U1RXREG;

	u16_rxFifo1Head = 0;
	u16_rxFifo1Tail = 0;
	
	if (_OERR) _OERR = 0;
	RX_Error = 0;
	_U1RXIF = 0;         //clear the UART RX interrupt bit
}

void RestoreNormalConnection()
{
	// -------------------------------------------------- ensure normal connection
	_CN12PUE = 0;	 // remove UART-TX weak-pull-up
	SLAVE_ON = 1;	 // reconnect the output line
	_UTXEN = 0;		 // disable TX pin
}

void InitAll()
{
	RestoreNormalConnection();
	CapStatus = INITIALIZING;
	Status = WAITING_START;
	_T1IE = 0;		// Disable Timer1 interrupt
}

// ------------------------------------ returns 1 if success / 0 if timeout
uint8 WaitAbaudZero()
{
	int32 timeoutcounter = 5000000; // about 5000 mS
	while (timeoutcounter-- > 0) 
	{
		if (_ABAUD == 0) return 1;
	}
	return 0;
}

// ----------------------------------------------------------------------------------------
//  Returns 1 if success / 0 if timeout - Used by Polling Mode
// ----------------------------------------------------------------------------------------
uint8 WaitUrxdaNotZero()
{
	int32 timeoutcounter = 500000; // about 500 mS
	while (timeoutcounter-- > 0) 
	{
		ClrWdt();
		//if (_FERR) return 1;
		if (_URXDA != 0) return 1;
	}
	Status = WAITING_START;
	return 0;
}	

// ----------------------------------------------------------------------------------------
//  Wait for a byte to be available from UART1 RX and return it - Used by Interrupt Mode
//  This function has not a Timeout - Ends only with a char or a gResync
// ----------------------------------------------------------------------------------------
uint8 UART_GetChar(void) 
{
	while (u16_rxFifo1Head == u16_rxFifo1Tail)
	{
		 ClrWdt();
		 if (gResync) return 0;
	}		 
  	u16_rxFifo1Tail++;
  	if (u16_rxFifo1Tail == UART1_RX_FIFO_SIZE) u16_rxFifo1Tail=0; 	// wrap if needed
  	return au8_rxFifo1[u16_rxFifo1Tail]; 							// return the character
}



// ==============================================================================================
//   UART - RX and TX
// ==============================================================================================

void BaudRateManagement()
{
	uint16 OldU1BRG;
	uint8 Speed;
	uint8 crc;
	
	// LED_ON   
	
	if (U1RXREG == 0)
	{
		OldU1BRG = U1BRG;
		_WAKE = 1;
		_ABAUD = 1;
		// --------------------------------- 
		if (WaitAbaudZero() == 0) return;

		// --------------------------------- ensure a minumum baud rate of 1000 bps
		if (U1BRG > 3999) U1BRG = 3999;
					
		// --------------------------------- wait the RX data available
		if (WaitUrxdaNotZero() == 0) return;
		Speed = U1RXREG;
		if (Speed < 1 || Speed > 12) RX_Error = 1;
		
		// --------------------------------- wait the RX data available
		if (WaitUrxdaNotZero() == 0) return;
		crc = U1RXREG;
		
		// --------------------------------- test the crc
		InitCrc();
		AppendByteToCrc(CMD_SPEED);
		if (AppendByteToCrc(Speed) != crc) RX_Error = 1;
		
		InitAll();
	}
	
	if (RX_Error)
	{
		U1BRG = OldU1BRG;
	}
	
	// LED_OFF 
}



void checkRxErrorUART1(void) 
{
	//uint8 u8_c;
	//check for errors, reset if detected.
	if (U1STAbits.PERR) 
	{
	 	//u8_c = U1RXREG; //clear error
	 	RX_Error = 1;	// reportError("UART1 parity error\n");
	}
	if (U1STAbits.FERR) 
	{
		BaudRateManagement();
	 	//u8_c = U1RXREG; //clear error
	 	RX_Error = 1;	// reportError("UART1 framing error\n");
	 	gResync = 1;
	 	EmptyRxFifo();
	}
	if (U1STAbits.OERR) 
	{
		U1STAbits.OERR = 0; //clear error
		RX_Error = 1;	// reportError("UART1 overrun error\n");
		gResync = 1;
	 	EmptyRxFifo();
	}
}


void __attribute__((__interrupt__, __no_auto_psv__, __shadow__)) _U1RXInterrupt(void)
{
	int8 u8_c;
	
	//_U1RXIF = 0;         //clear the UART RX interrupt bit
	
	//do
	{
		checkRxErrorUART1();
		u8_c = U1RXREG;      //read character
		u16_rxFifo1Head++;   //increment head pointer
		if (u16_rxFifo1Head == UART1_RX_FIFO_SIZE) u16_rxFifo1Head = 0; // wrap if needed
		if (u16_rxFifo1Head == u16_rxFifo1Tail) RX_Error = 1;	 //FIFO overrun!, report error
		au8_rxFifo1[u16_rxFifo1Head] = u8_c; //place in buffer
	}
	//while (_URXDA);
	
	_U1RXIF = 0;         //clear the UART RX interrupt bit
}



uint8 ReceiveByte()
{
	uint8 value;
	
	// ============================================================================== INTERRUPT MODE
	#if UART_RECEIVE_INTERRUPT == 0
		// --------------------------------- restore the normal fifo beaviour
		if (_OERR) _OERR = 0;
		//if (_FERR) _FERR = 0;
		RX_Error = 0;
	
		// --------------------------------- wait the RX data available
		if (WaitUrxdaNotZero())
		{
			// --------------------------------- if FRAME-ERROR then wait the auto-baud byte ( 0x55 )
			if (_FERR)
			{
				//_FERR = 0;
				BaudRateManagement();
				gResync = 1;
				return 0;
			}	
	
			// --------------------------------- receive value
			value = U1RXREG;
		}
		else
		{
			value = 0;
		}		
	#endif
	
	// ============================================================================== POLLING MODE
	#if UART_RECEIVE_INTERRUPT != 0
		if (_OERR) _OERR = 0;
		//if (_FERR) _FERR = 0;
		RX_Error = 0;
	
		value = UART_GetChar();
		if (RX_Error) 
		{
			RX_Error = 0;
			return 0;
		}	
	#endif

	// --------------------------------- return the byte
	return value;
}


void TxByte(uint8 value)
{
	//LED_ON
	
	//_U1RXIE = 0; 
	
	// ------------------------------- wait a little time before start transmission
	//Delay_us(500);
	Delay_us(U1BRG >> 2);

	// ------------------------------- enable TX pin
	_UTXEN = 1;	// enable TX pin

	// ------------------------------- send byte 
	U1TXREG = value;
	
	// ------------------------------- wait the TransmitShiftRegister empty 
	uint16 timeoutcounter = 65000;
	while (timeoutcounter-- > 0)
	{
		if (_TRMT == 1) break;
		if (gResync) break;
	}

	// ------------------------------- disable TX pin
	_UTXEN = 0;
	
	
	//_U1RXIE = 1; // only if interrupt enabled !!!
	
	// ------------------------------- 
	EmptyRxFifo();
	
	
	//LED_OFF
}


void Tx3Bytes(uint32 value)
{
	TxByte(value >> 16);
	TxByte(value >> 8);
	TxByte(value);
}





// ==============================================================================================
//   LOW PRIORITY ACTIVITIES
// ==============================================================================================

void LowPriorityActivities()
{
	// ---------------------------------------------------------- reset watchdog timer
	ClrWdt();
	
	// ---------------------------------------------------------- LowPriority START
	//LED_ON
	
	// ========================================================================
	//   CAP-SENSOR STATE MACHINE
	// ========================================================================
	switch (CapStatus)
	{
		case INITIALIZING:
		{
			// -------------------------------------------------- reset timer 2/3
			T2CONbits.TON = 0;
			TMR2 = 0;
			TMR3 = 0;

			// -------------------------------------------------- start Timer1 very near to the overflow
			TMR1 = 0xFFF8;
			T1CONbits.TON = 1;

			// -------------------------------------------------- using interrupt to start Timers 2/3 
			//          										  exactly when the Timer1 overflows
			_T1IF = 0;	// Clear Timer1 Interrupt Flag
			_T1IE = 1;	// Enable Timer1 interrupt

			CapStatus = WAITING_EXTERNAL_SYNC_FOR_START;
		}
		break;

		case WAITING_EXTERNAL_SYNC_FOR_START:
		{
			// -------------------------------------------------- wait timer1 interrupt completion 
			if (_T1IE == 0)
			{
				// ---------------------------------------------- re-enable Timer1 interrupt immediatly
				//												   allowing times from 1 to 30 mS 
				_T1IF = 0;
				_T1IE = 1;

				CapStatus = RUNNING;
			}
		}
		break;
		
		default:
		break;
	}
	
	// ---------------------------------------------------------- LowPriority END
	//LED_OFF	
}	





// ==============================================================================================
//   MAIN
// ==============================================================================================

int main( void )
{
	// ---------------------------------------------- all pins digital ( not AD )
	AD1PCFG = 0xFFFF;
	
	// -------------------------- To safely set the CPU IPL, use SET_CPU_IPL( 0 to 7 )
	//SET_CPU_IPL(0);

	// -------------------------- timer 1 to max interrupt priority	
	//IPC0 = 0b0111000000000000;

	// -------------------------- 
	//INTCON1 = 0;

	LATA = 0;
	LATB = 0;

	TRISA = 0b1111111111111111;		// all inputs
	TRISB = 0b0111111011111111;		// RB15(SLAVE_ON) RB8(LED) enabled-out-tristate

	// --------------------------------------------------------------------------- PULLUPS
	CNPU1 = 0b0110000100101101;		//  CN14(RB12) CN13(RB13) CN8(RA6) CN5(RB1) CN3(RA1) CN2(RA0) CN0(RA4)
	CNPU2 = 0b0000000000100000;		//  CN21(RB9)



	// ---------------------------------------------- set initial pin type
	PinType = UNUSED;
	
	
	// -----------------------------------------------
	//  INTERRUPT PRIORITIES
	// -----------------------------------------------
	//
	// INITIALIZATION
	// 1. Set the NSTDIS Control bit (INTCON1<15>) if nested interrupts are not desired.
	// 2. Select the user-assigned priority level for the interrupt source by writing the control bits
	//     in the appropriate IPCx register.
	// 3. Clear the interrupt flag status bit associated with the peripheral in the associated IFSx register.
	// 4. Enable the interrupt source by setting the interrupt enable control bit associated with the
	//     source in the appropriate IECx register.
	//
	// If multiple priority levels are not desired the IPCx register control bits for all
	// enabled interrupt sources may be programmed to the same non-zero value.
	//
	// At a device Reset, the IPCx registers ar initialized, 
	// such that all user interrup sources are assigned to priority level 4.
	//
	// ----------------------------------------------------
	// 7 = Hi / 1 = Low / 0 = disabled
	// ----------------------------------------------------
	//	_U1RXIP = 7;	// Uart1  ( serial receive interrupt )	( default = 4 )
	//	_T3IP = 3;  	// Timer3 ( servo and pwm )				( default = 4 )
	//	_T2IP = 4;  	// Timer2 ( fast counter )				( default = 4 )
	//	_INT2IP = 4;	// Int2   ( pin9 interrupt )			( default = 4 )
	//	_CNIP = 4;  	// CnInt  ( int on change )				( default = 4 )
	//	_NSTDIS = 1;	// 1 = interrupt nesting is disabled	( default = 0 )
	
	
	
	// =================================================================================================
	//  REFOCON
	// =================================================================================================

	// bit 15 ROEN: Reference Oscillator Output Enable bit
	//  1 = Reference oscillator enabled on REFO pin
	//  0 = Reference oscillator disabled

	// bit 14 Unimplemented: Read as 0
	
	// bit 13 ROSSLP: Reference Oscillator Output Stop in Sleep bit
	//  1 = Reference oscillator continues to run in Sleep
	//  0 = Reference oscillator is disabled in Sleep

	// bit 12 ROSEL: Reference Oscillator Source Select bit
	//  1 = Primary oscillator used as the base clock(1)(2)
	//  0 = System clock used as the base clock; base clock reflects any clock switching of the device

	// bit 11-8 RODIV3:RODIV0: Reference Oscillator Divisor Select bits
	//  1111 = Base clock value divided by 32,768
	//  1110 = Base clock value divided by 16,384
	//  1101 = Base clock value divided by 8,192
	//  1100 = Base clock value divided by 4,096
	//  1011 = Base clock value divided by 2,048
	//	1010 = Base clock value divided by 1,024
	//	1001 = Base clock value divided by 512
	//	1000 = Base clock value divided by 256
	//	0111 = Base clock value divided by 128
	//	0110 = Base clock value divided by 64
	//	0101 = Base clock value divided by 32
	//	0100 = Base clock value divided by 16
	//	0011 = Base clock value divided by 8
	//	0010 = Base clock value divided by 4
	//	0001 = Base clock value divided by 2
	//	0000 = Base clock value

	// bit 7-0 Unimplemented: Read as 0

	//	Note 1: The crystal oscillator must be enabled using the FOSC<2:0> bits; crystal maintains operation in Sleep mode
	//  Note 2: If ROSEL = 1 then the primary oscillator inverter is always ON

	//REFOCON = 0b1000111100000000;   // internal clock ( 32 Mhz )   / 32768 = 976 Hz
	//REFOCON = 0b1001111100000000;   // external clock ( 2.83 Mhz ) / 32768 = 86.4 Hz
	//REFOCON = 0b1001101000000000;   // external clock ( 2.83 Mhz ) / 1024 = 2.76 KHz



	// =================================================================================================
	//  CLKDIV : INTERNAL-FRC divisor + CPU-CLOCK divisor
	// =================================================================================================
	
	// bit 15 ROI: Recover on Interrupt bit
	// 1 = Interrupts clear the DOZEN bit and reset the CPU and peripheral clock ratio to 1:1
	// 0 = Interrupts have no effect on the DOZEN bit
	
	// bits 14/13/12 - CPU-CLOCK divisor
	// 111 = 1:128
	// 110 = 1:64
	// 101 = 1:32
	// 100 = 1:16
	// 011 = 1:8
	// 010 = 1:4
	// 001 = 1:2
	// 000 = 1:1
	
	// bit 11 DOZEN: DOZE Enable bit 
	//  (This bit is automatically cleared when the ROI bit is set and an interrupt occurs)
	// 1 = DOZE<2:0> bits specify the CPU and peripheral clock ratio
	// 0 = CPU and peripheral clock ratio set to 1:1
	
	// bit 10-8 RCDIV<2:0>: FRC Postscaler Select bits
	// 	 When OSCCON (COSC<2:0>) = 111:
	//  	111 = 31.25 kHz (divide by 256)
	//  	110 = 125 kHz (divide by 64) 
	//  	101 = 250 kHz (divide by 32) 
	//  	100 = 500 kHz (divide by 16) 
	//  	011 = 1 MHz (divide by 8) 
	//  	010 = 2 MHz (divide by 4) 
	//  	001 = 4 MHz (divide by 2) (default)
	//  	000 = 8 MHz (divide by 1)
	// 	 When OSCCON (COSC<2:0>) = 110:
	//  	111 = 1.95 kHz (divide by 256)
	//  	110 = 7.81 kHz (divide by 64) 
	//  	101 = 15.62 kHz (divide by 32) 
	//  	100 = 31.25 kHz (divide by 16) 
	//  	011 = 62.5 kHz (divide by 8) 
	//  	010 = 125 kHz (divide by 4) 
	//  	001 = 250 kHz (divide by 2) (default)
	//  	000 = 500 kHz (divide by 1)
	
	// bit 7-0 Unimplemented: 

    CLKDIV = 0;		// all divisors to 1:1 



	// =================================================================================================
	//  TIMER 1 - TESTS
	// =================================================================================================

	//_T1IF = 0;	// Clear Timer1 Interrupt Flag
	//_T1IE = 1;	// Enable Timer1 interrupt

	//T1CON = 0b0000000000000010;			// STOP Timer1 - 16bit - Max speed - no-sync - from pin T1CK
	//TMR1 = 0;
	//PR1 = 0xFFFE;

	// ------------------------------------------------ da frequenzimetro Mullins
	//AD1PCFG = 0x0020;
	//PR1 = 0xFFFE;
	//T1CON = 0x8002;	//  External asynchronous count/1, start TMR1, external clock at T1CK

	// ------------------------------------------------ BUILTIN WRITE
	//__builtin_write_OSCCONL(OSCCON & 0b01111111);		// unlock
	//__builtin_write_OSCCONH( 0b00110011);				// HSPLL
	//__builtin_write_OSCCONL(OSCCON | 0b00000010);		// soscen = 1
	//__builtin_write_OSCCONL(OSCCON & 0b11111101);		// soscen = 0

	//T1CON = 0x8012; 	//Start Timer1 with prescaler 1:8 and clock source external clock in asynchronous mode

	//	T1CONbits.TON = 1;		// 1 = ON  /  2 = OFF
	//	T1CONbits.TSIDL = 0;	// 1 = discontinued module operation in idle mode
	//	T1CONbits.TGATE = 0;	// 1 = gated accumulation enabled ( ignored if TCS = 1 )
	//	T1CONbits.TCKPS = 0;	// prescaler ( 0 = 1:1  /  1 = 1:8  /  2 = 1:64  /  3 = 1:256 )
	//	T1CONbits.TSYNC = 0;	// 1 = syncronize external clock  ( ignored if TCS = 0 )
	//	T1CONbits.TCS = 1;		// 0 = internal clock(fosc/2)  /  1 = External clock from pin T1CK(rising edge)

	//T1CON = 0b1000000000000010;			// Timer1 - 16bit - Max speed - no-sync - from pin T1CK
	//T1CON = 0b1000000000110110;			// Timer1 - 16bit - min speed - sync - from pin T1CK
	//T1CON = 0b1000000000000000;			// Timer1 - 16bit - Max speed - no-sync - from internal clock



	// =================================================================================================
	//   T1CON
	// =================================================================================================

	//	bit 15 TON: Timer1 On bit
	//	  1 = Starts 16-bit Timer1
	//	  0 = Stops 16-bit Timer1

	//	bit 14 Unimplemented: Read as 0

	//	bit 13 TSIDL: Stop in Idle Mode bit
	//	  1 = Discontinue module operation when device enters Idle mode
	//	  0 = Continue module operation in Idle mode

	//	bit 12-7 Unimplemented: Read as 0

	//	bit 6 TGATE: Timer1 Gated Time Accumulation Enable bit
	//	  When TCS = 1: 
	//	    This bit is ignored.
	//	  When TCS = 0: 
	//	    1 = Gated time accumulation enabled
	//	    0 = Gated time accumulation disabled

	//	bit 5-4 TCKPS<1:0>: Timer1 Input Clock Prescale Select bits
	//	  11 = 1:256 
	//	  10 = 1:64
	//	  01 = 1:8 
	//	  00 = 1:1

	//	bit 3 Unimplemented: Read as 0

	//	bit 2 TSYNC: Timer1 External Clock Input Synchronization Select bit
	//	  When TCS = 1: 
	//	    1 =  Synchronize external clock input
	//	    0 =  Do not synchronize external clock input
	//	  When TCS = 0: 
	//	    This bit is ignored.

	//	bit 1 TCS: Timer1 Clock Source Select bit
	//	  1 =  External clock from T1CK pin (on the rising edge)
	//	  0 =  Internal clock (FOSC/2)

	//	bit 0 Unimplemented: Read as 0


	// -----------------------------------------------------------------------------
	//  OK 
	//   - il segnale deve entrare da SOSCI (pin 9)
	//   - la tensione sull'ingresso SOSCI aperto e' di circa 0.74 volt
  	//   - le tensioni all'ingresso SOSCI devono essere: 
    //           Basso = 0.3 volt o meno
	//            Alto = 1.1 volt o piu'
	//
	//   - conta sicuramente fino a 3 Mhz e oltre ( probabilmente fino a 50 MHz ) 
	//   - da SOSCO non esce niente ( nemmeno abilitando il TRISA-4 )
	// -----------------------------------------------------------------------------
	__builtin_write_OSCCONL(OSCCON | 0b00000010);	// soscen = 1
	PR1 = 0xFFFF;
	//T1CON = 0b1000000000000010;					// Timer1 - 16bit - Max speed - no-sync - from external clock
	T1CON = 0b1000000000000110;						// Timer1 - 16bit - Max speed - sync - from external clock



	// =================================================================================================
	//  TIMER 2/3 
	// =================================================================================================

	T2CON = 0b1000000000001000;			// Timer23 - 32bit - Max speed  - from internal clock(16 MHz)
	//T2CON = 0b1000000000001010;		// Timer23 - 32bit - Max speed - sync - from pin T2CK



	// =====================================================================================
	//   UART
	// =====================================================================================

	// ---------------------------------------------------------------------------------------------------
  	//   RECEIVING BREAK CHARACTERS
	// ---------------------------------------------------------------------------------------------------	
	// The wake-up feature is enabled by setting the WAKE bit (UxMODE <7>) = 1. 
	//
	// In this mode, the module will receive the Start bit, 
	//   data and the invalid Stop bit (which sets FERR),
	//   but the receiver will wait for a valid Stop bit before looking for the next Start bit. 
	//
	// It will not assume that the Break condition on the line is the next Start bit. 
	//
	// Break is regarded as a character containing all 0s with the FERR bit set. 
	//
	// The Break character is loaded into the buffer. 
	//
	// No further reception can occur until a Stop bit is received. 
	//
	// The WAKE bit will get cleared automatically once the Stop bit is received after the 13-bit Break character.
	//
	// Note that RIDLE goes high when the Stop bit has been received.
	//
	// The receiver will count and expect a certain number of bit times based on the values
	//   programmed in the PDSEL<1:0> (UxMODE<2:1>) and STSEL (UxMODE<0>) bits. 
	//
	// If the Break is longer than 13 bit times, the reception is considered complete after the number of
	//  bit times specified by the PDSEL and STSEL bits. The URXDA bit is set, FERR is set, zeros are
	//  loaded into the receive FIFO and interrupts are generated.
	//
	// If the wake-up feature is not set, WAKE (UxMODE <7>) = 0, Break reception is not special. The
	// Break will be counted as one character loaded into the buffer (all 0 bits) with FERR set.
	// ---------------------------------------------------------------------------------------------------


	// -------------------------------------------------------------------------------------
	//  The UTXEN bit should not be set until the UARTEN bit has been set;
	//    otherwise,UART transmissions will not be enabled.
	//
	//	Disabling the UART module resets the buffers to empty states. 
	//    Any data characters in the buffers are lost and the baud rate counter is reset.
	// -------------------------------------------------------------------------------------

	// ------------------------- set baud generator to HiSpeed
	_BRGH = 1;
	// ------------------------- default com speed "7" ( 10 uS - 100KHz ) ( U1BRG = 4 MHz / BaudRate - 1 )
	U1BRG = 39;

	// ------------------------- TX and RX Interrupt Priority
	//IPC3bits.U1TXIP2 = 1;
	//IPC3bits.U1TXIP1 = 0;
	//IPC3bits.U1TXIP0 = 0;
	//IPC2bits.U1RXIP2 = 1;
	//IPC2bits.U1RXIP1 = 0;
	//IPC2bits.U1RXIP0 = 0;
	// ------------------------- enable Transmit Interrupt
	//IEC0bits.U1TXIE = 1;
	
	// ------------------------- enable Receive Interrupt
	#if UART_RECEIVE_INTERRUPT != 0
		//_U1RXIP = 1; 			//choose a priority
		_U1RXIF = 0;            //clear the flag
		_U1RXIE = 1;            //enable the interrupt
	#endif

	// ------------------------- mode = 8 N 1
	//U1STA = 0;
	//U1MODE = 0x8000;

	// ------------------------- enable UART and disable TX pin 
	_UARTEN = 1;
	_UTXEN = 0;

	// ------------------------- 00 = Interrupt is set when any character is received and transferred
	//                            from the RSR to the receive buffer.
	//                            Receive buffer has one or more characters.
	_URXISEL1 = 0;
	_URXISEL0 = 0;

	// ------------------------ 00 = Interrupt when a character is transferred to the Transmit Shift Register
	//                           (this implies there is at least one character open in the transmit buffer)
	// 							01 = Interrupt when the last character is shifted out of the Transmit Shift Register;
	//                           all transmit operations are completed
	_UTXISEL1 = 0;
	_UTXISEL0 = 1;

	// ---------------------------------------------------------------- set Slave as not recognized
	gSlaveId = INVALID_SLAVE_ID;
	
	// ---------------------------------------------------------------- wait UART line stabilization
	InitAll();
	Delay_ms(50);
	EmptyRxFifo();	



	// =================================================================================================
	//    MAIN LOOP
	// =================================================================================================

	uint8 CommandId;

	while(1)
	{
		
		// ---------------------------------------------------------- BLINK LED	
		static uint16 BlinkLedCounter = 0;
		if(BlinkLedCounter-- == 0)
		{
			BlinkLedCounter = 20U;
			LED = !LED;		
		}	
		
		// ---------------------------------------------------------- CapSensor restart - Must be called frequently
		LowPriorityActivities();     

		// ---------------------------------------------------------- MAIN STATE MACHINE
		switch (Status)
		{
			case WAITING_START:
			{
				gResync = 0;
				
				// -------------------------------------------------- COMMANDS RECEPTION AND MANAGEMENT
				CommandId = ReceiveByte();

				switch (CommandId)
				{
					// -------------------------- COMMAND - no action ( only sync )
					case CMD_NOCMD:
					{
						RestoreNormalConnection();
					}
					break;

					// -------------------------- COMMAND - start recognizing and enumeration
					case CMD_RECOG_START:
					{
						uint8 nBytes = ReceiveByte();
						uint8 crc = ReceiveByte();
						InitCrc();
						AppendByteToCrc(CMD_RECOG_START);
						uint8 mycrc = AppendByteToCrc(nBytes);
						
						if(mycrc == crc)
						{
							// ------------------------------------------------- break connection to other slaves
							_CN12PUE = 1;		// set UART-TX weak-pull-up
							SLAVE_ON = 0;		// disconnect the output line
							// ------------------------------------------------- 
							Status = RECOGNIZING;   
							// ------------------------------------------------- set Slave as not recognized
							gSlaveId = INVALID_SLAVE_ID;
						}
					}
					break;
					
					
					// -------------------------- COMMAND - setup slave pins
					case CMD_SETUP_SLAVE_PINS:
					{
						uint8 TargetSlave = ReceiveByte(); 
						uint8 nBytes = ReceiveByte();
						
						uint8 i;
						for (i = 0; i < nBytes; i++)
						{
							BytesFromMaster[i] = ReceiveByte();
						}
						
						uint8 crc = ReceiveByte();
						
						InitCrc();
						AppendByteToCrc(CMD_SETUP_SLAVE_PINS);
						AppendByteToCrc(TargetSlave);
						AppendByteToCrc(nBytes);
						uint8 mycrc = AppendArrayToCrc(&BytesFromMaster[0], nBytes);
					
						if (TargetSlave == gSlaveId)
						{
							if (crc == mycrc)
							{
								// Compute CRC only if success / otherwise leave it wrong
								InitCrc();
								mycrc = AppendByteToCrc(TargetSlave); 
								
								PinType = BytesFromMaster[0];
							}	
							TxByte(gSlaveId);
							TxByte(mycrc);
						}
						else
						{
							ReceiveByte();
							ReceiveByte();
						}	
					}
					break;
					
					// -------------------- COMMAND - send values to slave ( never used in this CapSensorHQ slave )
					case CMD_SEND_VALUES:
					{
						uint8 TargetSlave = ReceiveByte(); 
						uint8 nBytes = ReceiveByte();
						
						uint8 i;
						for (i = 0; i < nBytes; i++)
						{
							BytesFromMaster[i] = ReceiveByte();
						}
						
						uint8 crc = ReceiveByte();
						
						InitCrc();
						AppendByteToCrc(CMD_SEND_VALUES);
						AppendByteToCrc(TargetSlave);
						AppendByteToCrc(nBytes);
						uint8 mycrc = AppendArrayToCrc(&BytesFromMaster[0], nBytes);
					
						if (TargetSlave == gSlaveId)
						{
							if (crc == mycrc)
							{
								// Compute CRC only if success / otherwise leave it wrong
								InitCrc();
								mycrc = AppendByteToCrc(TargetSlave); 
								
								// --------------------------------------- do something with received data
								//for (i = 0; i < nBytes / 2; i++)
								//{
								//	ServoValue[i] = BytesFromMaster[i*2] * 256 + BytesFromMaster[i*2+1];
								//}	
							}	
							TxByte(gSlaveId);
							TxByte(mycrc);
						}
						else
						{
							ReceiveByte();
							ReceiveByte();
						}	
					}
					break;
					
					
					// -------------------------- COMMAND  "Get values from slave"
					case CMD_GET_VALUES:
					{
						uint8 i;
						uint8 TargetSlave = ReceiveByte(); 
						uint8 nBytes = ReceiveByte();
						uint8 crc = ReceiveByte();
						
						InitCrc();
						AppendByteToCrc(CMD_GET_VALUES);
						AppendByteToCrc(TargetSlave);
						uint8 mycrc = AppendByteToCrc(nBytes);
						uint8 crcerror = mycrc != crc;
						
						if (TargetSlave == gSlaveId)
						{					
							// --------------- TODO - IMPLEMENT the SEMAPHORE or the ATOMIC COMPARE
							// Tx3Bytes(T2_CapSensorTime);
							BytesToMaster[0] = (uint8)(T2_CapSensorTime >> 16);
							BytesToMaster[1] = (uint8)(T2_CapSensorTime >> 8);
							BytesToMaster[2] = (uint8)(T2_CapSensorTime);
						
							// ------------------------------------------ trasmit BYTES and SlaveId
							for (i = 0; i < nBytes; i++)
							{
								TxByte(BytesToMaster[i]);
							}
							TxByte(gSlaveId);
							// ------------------------------------------ calc CRC from BYTES and SlaveId
							InitCrc();
							AppendArrayToCrc(&BytesToMaster[0], nBytes);
							uint8 crc2 = AppendByteToCrc(gSlaveId);
							// ------------------------------------------ if crcerror from master then invalidate the crc
							if (crcerror) crc2 = crc2 ^ 255;
							// ------------------------------------------ trasmit the CRC
							TxByte(crc2);
						}
						else
						{
							for (i = 0; i < nBytes + 2; i++)
							{
								ReceiveByte();
							}
						}	
					}
					break;
					
					
					
					// -------------------------- COMMAND "Send bytes to slave"
					case CMD_SEND_BYTES:
					{
						uint8 TargetSlave = ReceiveByte(); 
						uint8 nBytes = ReceiveByte();
						
						uint8 i;
						for (i = 0; i < nBytes; i++)
						{
							BytesFromMaster[i] = ReceiveByte();
						}
						
						uint8 crc = ReceiveByte();
						
						InitCrc();
						AppendByteToCrc(CMD_SEND_BYTES);
						AppendByteToCrc(TargetSlave);
						AppendByteToCrc(nBytes);
						uint8 mycrc = AppendArrayToCrc(&BytesFromMaster[0], nBytes);
						
						if (TargetSlave == gSlaveId)
						{
							if (crc == mycrc)
							{
								// Compute CRC only if success / otherwise leave it wrong
								InitCrc();
								mycrc = AppendByteToCrc(TargetSlave); 
								
								// -------------------------------------------- use received values
								//for (i = 0; i < nBytes / 2; i++)
								//{
								//	ServoValue[i] = BytesFromMaster[i*2] * 256 + BytesFromMaster[i*2+1];
								//}	
							}	
							TxByte(gSlaveId);
							TxByte(mycrc);
						}
						else
						{
							ReceiveByte();
							ReceiveByte();
						}	
					}
					break;


					// -------------------------- COMMAND  "Get bytes from slave"
					case CMD_GET_BYTES:
					{
						uint8 i;
						uint8 TargetSlave = ReceiveByte(); 
						uint8 nBytes = ReceiveByte();
						uint8 crc = ReceiveByte();
						
						InitCrc();
						AppendByteToCrc(CMD_GET_BYTES);
						AppendByteToCrc(TargetSlave);
						uint8 mycrc = AppendByteToCrc(nBytes);
						uint8 crcerror = mycrc != crc;
						
						if (TargetSlave == gSlaveId)
						{
							// ------------------------------------------ fill BytesToMaster[] array
							// .....
						
							// ------------------------------------------ trasmit BYTES and SlaveId
							for (i = 0; i < nBytes; i++)
							{
								TxByte(BytesToMaster[i]);
							}
							TxByte(gSlaveId);
							// ------------------------------------------ calc CRC from BYTES and SlaveId
							InitCrc();
							AppendArrayToCrc(&BytesToMaster[0], nBytes);
							uint8 crc2 = AppendByteToCrc(gSlaveId);
							// ------------------------------------------ if crcerror from master then invalidate the crc
							if (crcerror) crc2 = crc2 ^ 255;
							// ------------------------------------------ trasmit the CRC
							TxByte(crc2);
						}
						else
						{
							for (i = 0; i < nBytes + 2; i++)
							{
								ReceiveByte();
							}
						}	
					}
					break;
					
				
				}//END switch (CommandId)

			}//END case WAITING_START:
			break;
			

			case RECOGNIZING:
			{
				CommandId = ReceiveByte();

				if (gSlaveId == INVALID_SLAVE_ID)
				{
					if (CommandId == CMD_RECOG)
					{
						// -------------------------- COMMAND - receive Slave-ID and send Slave-Type						

						uint8 nbytes = ReceiveByte();
						gSlaveId = ReceiveByte(); 
						uint8 crc = ReceiveByte();

						InitCrc();
						AppendByteToCrc(CMD_RECOG);
						AppendByteToCrc(nbytes);
						uint8 mycrc = AppendByteToCrc(gSlaveId);

						if (crc == mycrc)
						{
							TxByte(1);   		// 1 = Capacitive Sensor Hi Quality
							TxByte(2);   		// 2 = CRC of number "1"
							
							RestoreNormalConnection();
						}
					}
				}
			}
			break;
	
		}
	}
}

